package com.streamsicle.gui;

import com.streamsicle.*;
import edu.stanford.ejalbert.BrowserLauncher;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.apache.log4j.Category;
import com.jeans.trayicon.WindowsTrayIcon;
import com.jeans.trayicon.TrayIconCallback;
import com.jeans.trayicon.TrayIconPopup;
import com.jeans.trayicon.TrayIconPopupSimpleItem;

/**
 *  The main GUI of Streamsicle.
 *
 *@author     John Watkinson
 *@created    February 18, 2003
 */

public class MainWindow extends JFrame implements ActionListener, WindowListener {

    // -------------------------------------------------------------------------
    // CONSTANTS
    // -------------------------------------------------------------------------

    public final static String ACTION_CONFIGURE = "CONFIGURE";
    public final static String ACTION_BROWSER = "BROWSER";
    public final static String ACTION_SYSTRAY = "SYSTRAY";
    public final static String ACTION_QUIT = "QUIT";

    public final static Dimension WINDOW_SIZE = new Dimension(500, 200);

    public final static String LOGO_FILE = "web/img/logo.gif";
    public final static String ICON_FILE = "web/img/icons/megaphone.gif";

    // -------------------------------------------------------------------------
    // FIELDS
    // -------------------------------------------------------------------------

    private Streamsicle streamsicle;
    private Properties props;
    private String configFile;
    public WindowsTrayIcon icon;


    // -------------------------------------------------------------------------
    // CONSTANTS
    // -------------------------------------------------------------------------

    /**
     *  Constructor for the MainWindow object
     *
     *@param  configFile   Description of the Parameter
     *@param  streamsicle  Description of the Parameter
     */
    public MainWindow(String configFile, Streamsicle streamsicle) {
        super("Streamsicle Server");

        try {
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        this.configFile = configFile;
        this.streamsicle = streamsicle;
        props = streamsicle.getProperties();
        // Icon
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Image img = toolkit.getImage(ICON_FILE);
        setIconImage(img);
        String url = props.getProperty("streamsicle.host") + ":8080";
        JPanel topPanel = new JPanel(new BorderLayout());
        topPanel.setBackground(Color.white);
        JLabel glamLabel = new JLabel(new ImageIcon(LOGO_FILE));
        glamLabel.setBackground(Color.white);
        JLabel statusLabel = new JLabel("Streamsicle Server is running.");
        statusLabel.setBackground(Color.white);
        topPanel.add(glamLabel, BorderLayout.NORTH);
        topPanel.add(statusLabel, BorderLayout.SOUTH);
        JTextArea descriptionLabel = new JTextArea("Point your browser at http://" + url + " for the Streamsicle web interface.");
        descriptionLabel.setEditable(false);
        descriptionLabel.setOpaque(false);
        descriptionLabel.setWrapStyleWord(true);
        descriptionLabel.setLineWrap(true);
        descriptionLabel.setFont(new Font("Arial", Font.BOLD, 12));
        JButton configureButton = new JButton("Configure...");
        configureButton.setBackground(Color.white);
        JButton quitButton = new JButton("Shutdown");
        quitButton.setBackground(Color.white);
        JButton browserButton = new JButton("Open Browser");
        browserButton.setBackground(Color.white);
        JButton systrayButton = new JButton("To Systray");
        systrayButton.setBackground(Color.white);
        configureButton.setActionCommand(ACTION_CONFIGURE);
        browserButton.setActionCommand(ACTION_BROWSER);
        systrayButton.setActionCommand(ACTION_SYSTRAY);
        quitButton.setActionCommand(ACTION_QUIT);
        configureButton.addActionListener(this);
        browserButton.addActionListener(this);
        systrayButton.addActionListener(this);
        quitButton.addActionListener(this);
        JPanel buttonPanel = new JPanel(new FlowLayout());
        buttonPanel.setBackground(Color.white);
        buttonPanel.add(configureButton);
        buttonPanel.add(browserButton);
        buttonPanel.add(systrayButton);
        buttonPanel.add(quitButton);
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(topPanel, BorderLayout.NORTH);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        getContentPane().add(descriptionLabel, BorderLayout.CENTER);
        getContentPane().setBackground(Color.white);
        setBackground(Color.white);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(this);
        addWindowListener(new WindowClosingListener());

        try {
            // Set all the stuff up for the windows tray icon
            WindowsTrayIcon.setWindowsMessageCallback(new WindowsMessageCallback());
            WindowsTrayIcon.initTrayIcon("Streamsicle");
            icon = new WindowsTrayIcon(loadImage("megaphone16.gif"), 16, 16);
            icon.setToolTipText("Streamsicle");
            icon.setPopup(makePopup());
            icon.addActionListener(this);
            // This should allow the left click restore
            icon.addActionListener(new RestoreListener(false));
            icon.setVisible(false);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }


    /**
     *  Description of the Method
     */
    public void start() {
        setSize(WINDOW_SIZE);
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Dimension screenSize = toolkit.getScreenSize();
        Dimension size = getSize();
        int x = (screenSize.width - size.width) / 2;
        int y = (screenSize.height - size.height) / 2;
        setLocation(x, y);
        setVisible(true);
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    public void actionPerformed(ActionEvent e) {
        String action = e.getActionCommand();
        Category log = Category.getInstance(MainWindow.class);
        if (ACTION_CONFIGURE.equals(action)) {
            // Configure
            String dirString = props.getProperty("streamsicle.playdir");
            File dir = new File(dirString);
            String host = props.getProperty("streamsicle.host");
            String message = props.getProperty("streamsicle.description");
            String password = props.getProperty("streamsicle.adminPassword");
            ConfigurationWizard wizard = new ConfigurationWizard(configFile, this, dir, host, message, password);
            boolean success = false;
            try {
                success = wizard.run();
            } catch (Exception x) {
                log.error("Configuration Wizard crashed.");
                x.printStackTrace();
            }
            if (success) {
                // Rebristle
                streamsicle.reconfigure();
                String newDir = props.getProperty("streamsicle.playdir");
                if (!dirString.equals(newDir)) {
                    // Reload
                    streamsicle.doReload();
                }
            }
        } else if (ACTION_BROWSER.equals(action)) {
            String url = "http://" + props.getProperty("streamsicle.host") + ":8080";
            try {
                BrowserLauncher.openURL(url);
            } catch (IOException x) {
                log.error("Couldn't open browser: " + x);
            }
        } else if (ACTION_SYSTRAY.equals(action)) {
            // Send to systray if window visible
            if (isVisible()) {
                icon.setVisible(true);
                setVisible(false);
            }
        } else if (ACTION_QUIT.equals(action)) {
            // Shutdown
            if (confirmQuit()) {
                WindowsTrayIcon.cleanUp();
                System.exit(0);
            }
        }
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    protected boolean confirmQuit() {
        if (JOptionPane.showConfirmDialog(this, "Shutting down the server will disconnect all listeners. Proceed?", "Confirm Shutdown", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.YES_OPTION) {
            return true;
        } else {
            return false;
        }
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    public void windowClosing(WindowEvent e) {
        if (confirmQuit()) {
            WindowsTrayIcon.cleanUp();
            System.exit(0);
        }
    }


    // Required for WindowListener interface
    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    public void windowActivated(WindowEvent e) { }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    public void windowClosed(WindowEvent e) { }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    public void windowDeactivated(WindowEvent e) { }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    public void windowDeiconified(WindowEvent e) { }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    public void windowIconified(WindowEvent e) { }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    public void windowOpened(WindowEvent e) { }

    public static Image loadImage(String fileName) {
        try {
            File test = new File("web" + File.separator
                + "img" + File.separator + "icons" + File.separator + fileName);
            System.out.println("Icon file exists: "+test.exists());
            return Toolkit.getDefaultToolkit().getImage("web" + File.separator
                + "img" + File.separator + "icons" + File.separator + fileName);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    /**
     *  Make the menu that will show up when the tray icon is clicked
     *
     *@return    Description of the Return Value
     */
    public TrayIconPopup makePopup() {
        // Make new popup menu
        TrayIconPopup popup = new TrayIconPopup();
        // Add show, about, submenu, separator & exit item
        TrayIconPopupSimpleItem item = new TrayIconPopupSimpleItem("&Show");
        item.setDefault(true);
        // Each menu item can have it's own ActionListener
        item.addActionListener(new RestoreListener(true));
        popup.addMenuItem(item);
        // Add exit item
        item = new TrayIconPopupSimpleItem("E&xit");
        item.addActionListener(new ExitListener());
        popup.addMenuItem(item);
        return popup;
    }


    public class WindowClosingListener extends WindowAdapter {

        public void windowClosing(WindowEvent e) {
            System.out.println("Window close event");
            icon.setVisible(false);
        }
    }

    private class RestoreListener implements ActionListener {

        protected boolean from_menu;


        public RestoreListener(boolean fromMenu) {
            from_menu = fromMenu;
        }


        public void actionPerformed(ActionEvent evt) {
            // Make main window visible if it was hidden
            setVisible(true);
            icon.setVisible(false);
            // Request input focus
            requestFocus();
        }
    }

    private class ExitListener implements ActionListener {

        public void actionPerformed(ActionEvent evt) {
            windowClosing(null);
        }

    }

    private class WindowsMessageCallback implements TrayIconCallback {

        public int callback(int param) {
            // Param contains the integer value send with sendWindowsMessage(appName, param)
            System.out.println("Other instance started (parameter: " + param + ")");
            setVisible(true);
            requestFocus();
            // Return integer value to other process
            return 4321;
        }
    }

}
